﻿using Model.Utils;

using static Model.Utils.LowerFieldType;
using static Model.Utils.UpperFieldType;

namespace Model
{
    /// <summary>
    /// The BasicMonster is a subclass of the Monster class. It moves randomly.
    /// </summary>
    internal class BasicMonster : Monster
    {
        /// <summary>
        /// Initializes a new instance of the BasicMonster class.
        /// </summary>
        /// <param name="x">Initial X position.</param>
        /// <param name="y">Initial Y position.</param>
        /// <param name="indexer">The indexer function of the map.</param>
        public BasicMonster(int x, int y, Func<int, int, (LowerFieldType, UpperFieldType)> indexer) : base(x, y, indexer)
        {
            _moveTimer.Interval = 1500;
            _moveTimer.Start();
            _canStepOnTheseLowers = [LowerFieldType.Empty, MoreBombsEffect, BiggerBombsEffect, DetonatorEffect, SkatesEffect, InvincibilityEffect, GhostEffect, BarrierEffect, Debuff];
            _canStepOnTheseUppers = [UpperFieldType.Empty, BluePlayer, RedPlayer];
        }

        /// \cond DOXYGEN_SHOULD_SKIP_THIS
        /// <summary>
        /// Only for testing! Initializes a new instance of the Model.BasicMonster class for testing.
        /// </summary>
        internal BasicMonster(int x, int y, Func<int, int, (LowerFieldType, UpperFieldType)> indexer, int interval) : this(x, y, indexer)
        {
            _moveTimer.Interval = interval;
        }
        /// \endcond

        protected override Direction? DoDecideDirectionToMove()
        {
            // determining empty spaces around monster
            List<Direction> valids = GetValidDirections();

            if (valids.Count == 0)
            {
                // no neighbouring spaces are valid to move to
                return null;
            }

            Direction direction = _direction;

            if (valids.Count == 1)
            {
                // we can only move to one square
                direction = valids[0];
            }
            else
            {
                // we can move to at least two spaces

                if (valids.Contains(direction))
                {
                    // the square in the direction the monster's facing is valid

                    int rng = _random.Next(100);
                    if (rng < 30)
                    {
                        // we change direction with a 30% chance
                        valids.Remove(direction);
                        direction = valids[_random.Next(valids.Count)];
                    }
                }
                else
                {
                    // we need to change directions
                    direction = valids[_random.Next(valids.Count)];
                }
            }

            return direction;
        }

        #region IDisposable

        private bool _disposed = false;

        ~BasicMonster() => Dispose(false);

        protected override void Dispose(bool disposing)
        {
            if (!_disposed)
            {
                if (disposing)
                {

                }

                _disposed = true;

                base.Dispose(disposing);
            }
        }

        #endregion
    }
}
