﻿using Model.Utils;

using LFT = Model.Utils.LowerFieldType;
using UFT = Model.Utils.UpperFieldType;

namespace Model
{
    public partial class GameModel : IDisposable
    {
        private static readonly LFT E = LFT.Empty;
        private static readonly LFT I = LFT.InnerWall;
        private static readonly LFT O = LFT.OuterWall;
        private static readonly LFT B = LFT.Box;

        private static readonly UFT e = UFT.Empty;
        private static readonly UFT b = UFT.BluePlayer;
        private static readonly UFT r = UFT.RedPlayer;
        private static readonly UFT m = UFT.BasicMonster;

        /// <summary>
        /// The height of the layouts.
        /// </summary>
        public int HEIGHT { get; } = 15;

        /// <summary>
        /// The width of the layouts.
        /// </summary>
        public int WIDTH { get; } = 15;

        private readonly LowerFieldType[,] LayoutOneLowerLayer = new LowerFieldType[15, 15]
        {
           {O, O, O, O, O, O, O, O, O, O, O, O, O, O, O},
           {O, E, E, E, E, E, E, E, E, E, E, E, E, E, O},
           {O, E, I, B, I, E, I, B, I, E, I, B, I, E, O},
           {O, E, B, B, B, E, B, B, B, E, B, B, B, E, O},
           {O, E, I, B, I, E, I, B, I, E, I, B, I, E, O},
           {O, E, B, B, B, E, B, B, B, E, B, B, B, E, O},
           {O, E, E, E, E, E, E, E, E, E, E, E, E, E, O},
           {O, E, B, I, B, B, I, B, I, B, B, I, B, E, O},
           {O, E, E, E, E, E, E, E, E, E, E, E, E, E, O},
           {O, E, B, B, B, E, B, B, B, E, B, B, B, E, O},
           {O, E, I, B, I, E, I, B, I, E, I, B, I, E, O},
           {O, E, B, B, B, E, B, B, B, E, B, B, B, E, O},
           {O, E, I, B, I, E, I, B, I, E, I, B, I, E, O},
           {O, E, E, E, E, E, E, E, E, E, E, E, E, E, O},
           {O, O, O, O, O, O, O, O, O, O, O, O, O, O, O}
        };
        private readonly UpperFieldType[,] LayoutOneUpperLayer = new UpperFieldType[15, 15]
        {
           {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
           {e, b, e, e, e, e, e, e, e, e, e, e, e, e, e},
           {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
           {e, e, e, e, e, e, e, e, e, m, e, e, e, e, e},
           {e, e, e, e, e, m, e, e, e, e, e, e, e, e, e},
           {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
           {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
           {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
           {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
           {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
           {e, e, e, e, e, e, e, e, e, m, e, e, e, e, e},
           {e, e, e, e, e, m, e, e, e, e, e, e, e, e, e},
           {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
           {e, e, e, e, e, e, e, e, e, e, e, e, e, r, e},
           {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e}
        };

        private readonly LowerFieldType[,] LayoutTwoLowerLayer = new LowerFieldType[15, 15]
        {
            {O, O, O, O, O, O, O, O, O, O, O, O, O, O, O},
            {O, E, E, B, I, B, E, I, B, E, E, E, I, E, O},
            {O, I, E, E, E, E, E, I, E, E, B, E, B, E, O},
            {O, I, B, E, B, B, I, I, I, E, E, I, B, E, O},
            {O, I, I, B, E, B, I, I, E, B, E, B, E, E, O},
            {O, I, E, E, B, E, I, B, B, E, B, E, E, E, O},
            {O, B, B, B, E, E, E, E, E, E, E, B, I, E, O},
            {O, E, E, E, E, B, B, E, E, B, E, B, I, E, O},
            {O, E, B, E, I, E, B, I, I, E, E, B, I, B, O},
            {O, B, E, E, I, B, B, E, E, E, B, B, I, E, O},
            {O, E, E, E, I, E, E, B, E, I, B, E, B, E, O},
            {O, I, B, E, I, E, B, E, E, I, E, E, E, E, O},
            {O, I, B, E, B, E, B, E, I, I, I, E, E, E, O},
            {O, I, I, E, E, E, E, E, B, I, B, E, B, I, O},
            {O, O, O, O, O, O, O, O, O, O, O, O, O, O, O}
        };
        private readonly UpperFieldType[,] LayoutTwoUpperLayer = new UpperFieldType[15, 15]
        {
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, b, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, m, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, m, e, e, e, e, e, e, e},
            {e, e, e, e, e, m, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, r, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e}
        };

        private readonly LowerFieldType[,] LayoutThreeLowerLayer = new LowerFieldType[15, 15]
        {
            {O, O, O, O, O, O, O, O, O, O, O, O, O, O, O},
            {O, E, E, E, E, B, B, E, E, E, B, B, E, B, O},
            {O, B, I, E, I, B, I, E, I, B, I, B, I, B, O},
            {O, E, E, E, E, B, B, E, B, E, B, E, E, B, O},
            {O, B, I, B, I, B, I, B, I, B, I, B, I, B, O},
            {O, E, B, E, B, E, E, E, B, E, B, E, B, E, O},
            {O, E, I, B, I, E, I, B, I, E, I, B, I, E, O},
            {O, E, E, E, B, B, E, E, E, B, B, B, E, B, O},
            {O, B, I, B, I, B, I, E, I, B, I, B, I, B, O},
            {O, B, E, E, B, B, E, E, B, B, B, E, B, B, O},
            {O, B, I, B, I, E, I, B, I, B, I, E, I, B, O},
            {O, E, E, E, B, E, B, E, E, B, B, E, E, E, O},
            {O, E, I, E, I, B, I, E, I, B, I, B, I, E, O},
            {O, E, E, E, B, B, B, E, B, E, E, E, B, E, O},
            {O, O, O, O, O, O, O, O, O, O, O, O, O, O, O}
        };
        private readonly UpperFieldType[,] LayoutThreeUpperLayer = new UpperFieldType[15, 15]
        {
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, b, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, m, e, e, e, m, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, m, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, m, e, e, e, e, e, e, e},
            {e, e, e, m, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e},
            {e, e, e, e, e, e, e, m, e, e, e, e, e, e, e},
            {e, m, e, e, e, e, e, e, e, e, e, e, e, r, e},
            {e, e, e, e, e, e, e, e, e, e, e, e, e, e, e}
        };
    }
}
