﻿using Timer = System.Timers.Timer;

namespace Model
{
    /// <summary>
    /// Defines the basis for effects of which multiple can be applied to the same target at the same time.
    /// </summary>
    internal class StackableEffect : IDisposable
    {
        private Action _applyEffect;
        private Action _removeEffect;
        private Timer _timer;

        /// <summary>
        /// Initializes a new instance of the Model.StackableEffect class.
        /// </summary>
        /// <param name="interval">The time, in milliseconds, which the effect lasts for.</param>
        /// <param name="apply">The Action that applies the effect.</param>
        /// <param name="remove">The Action that reverses the effect.</param>
        public StackableEffect(int interval, Action apply, Action remove)
        {
            _applyEffect = apply;
            _removeEffect = remove;

            _applyEffect();
            _timer = new Timer(interval);
            _timer.Elapsed += (sender, e) => _removeEffect();
            _timer.AutoReset = false;
            _timer.Start();
        }

        #region IDisposable

        private bool _disposed = false;

        ~StackableEffect() => Dispose(false);

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!_disposed)
            {
                if (disposing)
                {
                    _timer.Dispose();
                }

                _disposed = true;
            }
        }

        #endregion
    }
}
