﻿namespace Model.Utils
{
    /// <summary>
    /// Provides data for events that describe a step from one space to another.
    /// </summary>
    public class MoveEventArgs : EventArgs
    {
        /// <summary>
        /// The X position of the tile the move is originated from.
        /// </summary>
        public int FromX { get; }

        /// <summary>
        /// The Y position of the tile the move is originated from.
        /// </summary>
        public int FromY { get; }

        /// <summary>
        /// The X position of the tile the move is heading to.
        /// </summary>
        public int ToX { get; }

        /// <summary>
        /// The Y position of the tile the move is heading to.
        /// </summary>
        public int ToY { get; }

        /// <summary>
        /// The direction the move is in.
        /// </summary>
        public Direction Direction { get; }

        /// <summary>
        /// Initializes a new instance of the Model.Utils.MoveEventArgs class.
        /// </summary>
        /// <param name="fromX">The X position of the tile the move is originated from.</param>
        /// <param name="fromY">The Y position of the tile the move is originated from.</param>
        /// <param name="toX">The X position of the tile the move is heading to.</param>
        /// <param name="toY">The Y position of the tile the move is heading to.</param>
        /// <param name="direction">The direction the move is in.</param>
        public MoveEventArgs(int fromX, int fromY, int toX, int toY, Direction direction)
        {
            FromX = fromX;
            FromY = fromY;
            ToX = toX;
            ToY = toY;
            Direction = direction;
        }
    }
}
