﻿namespace Test
{
    [TestClass]
    public class BombTest
    {
        int x, y;
        int counter;

        private void TesterHandler(object? sender, EventArgs e)
        {
            ++counter;
            Assert.IsTrue(sender is Bomb);
        }

        [TestInitialize]
        public void Initialize()
        {
            x = 4;
            y = 2;
            counter = 0;
        }

        [TestMethod]
        [DataRow(2, 4)]
        [DataRow(0, 0)]
        [DataRow(-3, 43)]
        [DataRow(2132714927, 4726803)]
        public void ConstructorTest(int x, int y)
        {
            Bomb autoExploder = new(x, y, false);
            Assert.AreEqual(x, autoExploder.X);
            Assert.AreEqual(y, autoExploder.Y);
            Assert.IsFalse(autoExploder.IsDetonable);

            Bomb detonable = new(x, y, true);
            Assert.AreEqual(x, detonable.X);
            Assert.AreEqual(y, detonable.Y);
            Assert.IsTrue(detonable.IsDetonable);
        }

        [TestMethod]
        public void AutoExploderTest()
        {
            Bomb bomb = new(x, y, false, 100);
            bomb.BombIsExploding += TesterHandler;

            Thread.Sleep(50);
            Assert.AreEqual(0, counter);
            Thread.Sleep(100);
            Assert.AreEqual(1, counter);
            Thread.Sleep(100);
            Assert.AreEqual(1, counter);
        }

        [TestMethod]
        [DataRow(20)]
        [DataRow(0)]
        [DataRow(80)]
        public void ExplodeNowWithAutoExploderTest(int waitTime)
        {
            Bomb bomb = new(x, y, false, 100);
            bomb.BombIsExploding += TesterHandler;

            Thread.Sleep(waitTime);
            Assert.AreEqual(0, counter);
            bomb.ExplodeNow();
            Assert.AreEqual(1, counter);
            Thread.Sleep(100 - waitTime + 20);
            Assert.AreEqual(1, counter);
        }

        [TestMethod]
        [DataRow(20)]
        [DataRow(0)]
        [DataRow(100)]
        [DataRow(700)]
        [DataRow(1500)]
        public void DetonableTest(int waitTime)
        {
            Bomb bomb = new(x, y, true, 100);
            bomb.BombIsExploding += TesterHandler;

            Thread.Sleep(waitTime);
            Assert.AreEqual(0, counter);
            bomb.ExplodeNow();
            Assert.AreEqual(1, counter);
            Thread.Sleep(waitTime);
            Assert.AreEqual(1, counter);
        }
    }
}
