﻿namespace Test
{
    [TestClass]
    public class GameModelTest
    {
        private static readonly LowerFieldType E = LowerFieldType.Empty;
        private static readonly LowerFieldType O = LowerFieldType.OuterWall;

        private static readonly UpperFieldType e = UpperFieldType.Empty;
        private static readonly UpperFieldType b = UpperFieldType.BluePlayer;
        private static readonly UpperFieldType r = UpperFieldType.RedPlayer;

        private static readonly LowerFieldType[] allEffects = [LowerFieldType.Debuff, LowerFieldType.MoreBombsEffect, LowerFieldType.BiggerBombsEffect, LowerFieldType.DetonatorEffect, LowerFieldType.SkatesEffect, LowerFieldType.BarrierEffect, LowerFieldType.InvincibilityEffect, LowerFieldType.GhostEffect];

        private LowerFieldType[,] lowerLayer = null!;
        private UpperFieldType[,] upperLayer = null!;

        int counter;
        bool done;
        bool effect;

        private void TesterHandler(object? sender, EventArgs e)
        {
            ++counter;
        }

        private void TesterHandlerFieldChangedBomb(object? sender, FieldChangedEventArgs e)
        {
            if (e.Type == LowerFieldType.Bomb)
            {
                done = true;
            }
        }

        private void TesterHandlerFieldChangedEffect(object? sender, FieldChangedEventArgs e)
        {
            if (allEffects.Contains(e.Type) || e.Type == LowerFieldType.Empty)
            {
                effect = true;
            }
        }

        [TestInitialize]
        public void TestInitialize()
        {
            counter = 0;
            done = false;
            effect = false;
            lowerLayer = new LowerFieldType[7, 7]
            {
                { O, O, O, O, O, O, O },
                { O, E, E, E, E, E, O },
                { O, E, E, E, E, E, O },
                { O, E, E, E, E, E, O },
                { O, E, E, E, E, E, O },
                { O, E, E, E, E, E, O },
                { O, O, O, O, O, O, O }
            };
            upperLayer = new UpperFieldType[7, 7]
            {
                {e, e, e, e, e, e, e },
                {e, e, e, e, e, e, e },
                {e, e, b, e, e, e, e },
                {e, e, e, e, e, e, e },
                {e, e, e, e, r, e, e },
                {e, e, e, e, e, e, e },
                {e, e, e, e, e, e, e }
            };
        }

        [TestMethod]
        [DataRow(1)]
        [DataRow(2)]
        [DataRow(3)]
        public void NewCompetitionStartsTest(int layout)
        {
            GameModel model = new GameModel(lowerLayer, upperLayer, layout, 15);
            model.NewRound += TesterHandler;
            model.BattleRoyaleTimeChanged += TesterHandler;
            Assert.AreEqual(0, counter);
            model.NewCompetitionStarts(2, layout);
            Assert.AreEqual(2, counter);
        }

        [TestMethod]
        public void DeadPlayerMovesTest()
        {
            upperLayer[2, 3] = UpperFieldType.BasicMonster;
            GameModel model = new GameModel(lowerLayer, upperLayer, 1, 15);
            model.NewCompetitionStarts(2, 1);
            model.EntityDies += TesterHandler;
            Assert.AreEqual(0, counter);
            model.Move(PlayerType.Blue, Direction.Right);
            Assert.AreEqual(1, counter);
        }

        [TestMethod]
        public void AlivePlayerMovesTest()
        {
            GameModel model = new GameModel(lowerLayer, upperLayer, 1, 15);
            model.NewCompetitionStarts(2, 1);
            model.EntityMoved += TesterHandler;
            Assert.AreEqual(0, counter);
            model.Move(PlayerType.Blue, Direction.Right);
            Assert.AreEqual(1, counter);
        }

        [TestMethod]
        [DataRow(LowerFieldType.MoreBombsEffect)]
        [DataRow(LowerFieldType.BiggerBombsEffect)]
        [DataRow(LowerFieldType.DetonatorEffect)]
        [DataRow(LowerFieldType.SkatesEffect)]
        [DataRow(LowerFieldType.InvincibilityEffect)]
        [DataRow(LowerFieldType.GhostEffect)]
        [DataRow(LowerFieldType.BarrierEffect)]
        [DataRow(LowerFieldType.Debuff)]
        public void AlivePlayerMovesEffectTest(LowerFieldType lower)
        {
            lowerLayer[2, 3] = lower;
            GameModel model = new GameModel(lowerLayer, upperLayer, 1, 100);
            model.NewCompetitionStarts(2, 1);
            model.FieldChanged += TesterHandler;
            model.FieldChanged += TesterHandlerFieldChangedBomb;
            Assert.AreEqual(0, counter);
            model.Move(PlayerType.Blue, Direction.Right);
            Thread.Sleep(100);
            if (done)
            {

                Assert.AreEqual(2, counter);
            }
            else
            {
                Assert.AreEqual(1, counter);
            }
        }

        [TestMethod]
        public void PlayerInstantBombsMovesTest()
        {
            lowerLayer[2, 4] = LowerFieldType.MoreBombsEffect;
            GameModel model = null!;
            while (!done)
            {
                lowerLayer[2, 3] = LowerFieldType.Debuff;
                model = new GameModel(lowerLayer, upperLayer, 1, 100);
                model.NewCompetitionStarts(2, 1);
                model.FieldChanged += TesterHandler;
                model.FieldChanged += TesterHandlerFieldChangedBomb;
                model.Move(PlayerType.Blue, Direction.Right);
            }

            int currentCount = counter;
            Thread.Sleep(600);
            Assert.IsTrue(done);
            model.Move(PlayerType.Blue, Direction.Right);
            Thread.Sleep(600);
            Assert.AreEqual(currentCount + 2, counter);

            model.Move(PlayerType.Blue, Direction.Right);
            Assert.AreEqual(currentCount + 2, counter);
        }

        [TestMethod]
        public void PlaceBarriersTest()
        {
            lowerLayer[2, 3] = LowerFieldType.BarrierEffect;
            lowerLayer[4, 3] = LowerFieldType.BarrierEffect;
            GameModel model = new GameModel(lowerLayer, upperLayer, 1, 100);
            model.NewCompetitionStarts(2, 1);
            model.FieldChanged += TesterHandler;
            //BluePlayer
            model.PlaceBarrier(PlayerType.Blue);
            Assert.AreEqual(0, counter);

            model.Move(PlayerType.Blue, Direction.Right); // 2,3
            Assert.AreEqual(1, counter);
            model.PlaceBarrier(PlayerType.Blue);
            Assert.AreEqual(2, counter);

            //RedPlayer
            model.PlaceBarrier(PlayerType.Red);
            Assert.AreEqual(2, counter);

            model.Move(PlayerType.Red, Direction.Left); // 4,3
            Assert.AreEqual(3, counter);
            model.PlaceBarrier(PlayerType.Red); // első barrier
            Assert.AreEqual(4, counter);
            Thread.Sleep(600);
            model.Move(PlayerType.Red, Direction.Left); // 4,2
            model.PlaceBarrier(PlayerType.Red); // második barrier
            Assert.AreEqual(5, counter);
            Thread.Sleep(600);
            model.Move(PlayerType.Red, Direction.Up); // 3,2
            model.PlaceBarrier(PlayerType.Red); // harmadik barrier
            Assert.AreEqual(6, counter);
            Thread.Sleep(600);
            model.Move(PlayerType.Red, Direction.Up); // 2,2
            model.PlaceBarrier(PlayerType.Red); // nincs több barrier
            Assert.AreEqual(6, counter);
        }

        [TestMethod]
        public void BarrierExplodesTest()
        {
            lowerLayer[4, 3] = LowerFieldType.BarrierEffect;
            GameModel model = new GameModel(lowerLayer, upperLayer, 1, 100);
            model.NewCompetitionStarts(2, 1);
            model.FieldChanged += TesterHandler;

            model.PlaceBarrier(PlayerType.Red);
            Assert.AreEqual(0, counter);
            model.Move(PlayerType.Blue, Direction.Left); // 2,1

            model.Move(PlayerType.Red, Direction.Left); // 4,3
            Assert.AreEqual(1, counter);
            model.PlaceBarrier(PlayerType.Red); // első barrier
            Assert.AreEqual(2, counter);
            Thread.Sleep(600);
            model.Move(PlayerType.Red, Direction.Left); // 4,2
            model.PlaceBarrier(PlayerType.Red); // második barrier
            Assert.AreEqual(3, counter);
            Thread.Sleep(600);
            model.Move(PlayerType.Red, Direction.Up); // 3,2
            model.PlaceBarrier(PlayerType.Red); // harmadik barrier
            Assert.AreEqual(4, counter);
            Thread.Sleep(600);
            model.Move(PlayerType.Red, Direction.Up); // 2,2
            model.PlaceBarrier(PlayerType.Red); // nincs több barrier
            Assert.AreEqual(4, counter);
            Thread.Sleep(600);
            model.Move(PlayerType.Red, Direction.Right); // 2,3
            model.PlaceBomb(PlayerType.Red);
            Thread.Sleep(600);
            model.Move(PlayerType.Red, Direction.Right); // 2,4
            Thread.Sleep(600);
            model.Move(PlayerType.Red, Direction.Down); // 2,5
            Thread.Sleep(5000);
            Assert.AreEqual(7, counter);
            model.PlaceBarrier(PlayerType.Red);
            Assert.AreEqual(8, counter);
        }

        [TestMethod]
        public void DeadPlayerPlaceBarriersTest()
        {
            lowerLayer[2, 3] = LowerFieldType.BarrierEffect;

            GameModel model = new GameModel(lowerLayer, upperLayer, 1, 100);
            model.NewCompetitionStarts(2, 1);
            model.FieldChanged += TesterHandler;

            model.Move(PlayerType.Blue, Direction.Right); // 2,3
            Assert.AreEqual(1, counter);
            model.PlaceBarrier(PlayerType.Blue);
            Assert.AreEqual(2, counter);
            Thread.Sleep(600);
            model.Move(PlayerType.Blue, Direction.Left);
            model.PlaceBomb(PlayerType.Blue);
            Thread.Sleep(6000);
            Assert.AreEqual(5, counter);
            model.PlaceBarrier(PlayerType.Blue);
            Assert.AreEqual(5, counter);
        }

        [TestMethod]
        public void TerminateCompetitionTest()
        {
            lowerLayer[3, 1] = LowerFieldType.InnerWall;
            lowerLayer[3, 2] = LowerFieldType.InnerWall;
            lowerLayer[3, 3] = LowerFieldType.InnerWall;
            lowerLayer[4, 3] = LowerFieldType.InnerWall;
            lowerLayer[5, 3] = LowerFieldType.InnerWall;
            upperLayer[4, 1] = UpperFieldType.BasicMonster;
            lowerLayer[2, 4] = LowerFieldType.MoreBombsEffect;
            GameModel model = new GameModel(lowerLayer, upperLayer, 1, 100);
            model.FieldChanged += TesterHandler;
            model.EntityMoved += TesterHandler;
            model.FieldChanged += TesterHandlerFieldChangedBomb;
            model.NewCompetitionStarts(2, 1);
            model.PlaceBomb(PlayerType.Blue);
            Assert.IsTrue(done);
            counter = 0;
            model.TerminateCompetition();
            Thread.Sleep(6000);
            Assert.AreEqual(0, counter);
        }

        [TestMethod]
        public void GenerateRandomEffectTest()
        {
            GameModel model = null!;
            for (int i = 0; i < 4; i++)
            {
                lowerLayer[2, 3] = LowerFieldType.Box;
                model = new GameModel(lowerLayer, upperLayer, 3, 100);
                model.NewCompetitionStarts(2, 3);
                model.FieldChanged += TesterHandler;
                model.FieldChanged += TesterHandlerFieldChangedEffect;
                model.PlaceBomb(PlayerType.Blue);
                model.Move(PlayerType.Blue, Direction.Down);
                Thread.Sleep(600);
                model.Move(PlayerType.Blue, Direction.Right);
                effect = false;
                Thread.Sleep(6000);
                Assert.IsTrue(effect);
            }
        }

        [TestMethod]
        public void MonsterDiesBombTest()
        {
            upperLayer[2, 2] = UpperFieldType.Empty; // kék törlése
            lowerLayer[2, 1] = LowerFieldType.InnerWall;
            lowerLayer[2, 2] = LowerFieldType.InnerWall;
            lowerLayer[2, 3] = LowerFieldType.InnerWall;
            lowerLayer[2, 4] = LowerFieldType.InnerWall;
            upperLayer[3, 2] = UpperFieldType.BluePlayer;
            upperLayer[1, 1] = UpperFieldType.BasicMonster;
            lowerLayer[3, 3] = LowerFieldType.BiggerBombsEffect;
            lowerLayer[3, 4] = LowerFieldType.BiggerBombsEffect;
            lowerLayer[3, 5] = LowerFieldType.BiggerBombsEffect;
            lowerLayer[2, 5] = LowerFieldType.InvincibilityEffect;
            lowerLayer[1, 5] = LowerFieldType.MoreBombsEffect;
            GameModel model = new GameModel(lowerLayer, upperLayer, 1, 100);
            model.NewCompetitionStarts(2, 1);
            model.FieldChanged += TesterHandler;
            model.FieldChanged += TesterHandlerFieldChangedBomb;
            model.Move(PlayerType.Blue, Direction.Right); // 3,3
            Thread.Sleep(600);
            model.Move(PlayerType.Blue, Direction.Right); // 3,4
            Thread.Sleep(600);
            model.Move(PlayerType.Blue, Direction.Right); // 3,5
            Thread.Sleep(600);
            model.Move(PlayerType.Blue, Direction.Up); // 2,5 - Invincible
            Thread.Sleep(600);
            model.Move(PlayerType.Blue, Direction.Up);
            model.PlaceBomb(PlayerType.Blue);
            Thread.Sleep(6000);
            int currentCounter = counter;
            model.PlaceBomb(PlayerType.Blue);
            Thread.Sleep(6000);
            Assert.AreEqual(currentCounter + 2, counter);
        }

        [TestMethod]
        public void RoundOverTest()
        {
            GameModel model = new GameModel(lowerLayer, upperLayer, 1, 100);
            model.NewCompetitionStarts(2, 1);
            Assert.AreEqual(0, model.RedPoints);
            Assert.AreEqual(0, model.BluePoints);
            model.PlaceBomb(PlayerType.Blue);
            model.Move(PlayerType.Blue, Direction.Right);
            Thread.Sleep(600);
            model.Move(PlayerType.Blue, Direction.Down);
            model.PlaceBomb(PlayerType.Red);
            Thread.Sleep(12000);
            Assert.AreEqual(1, model.BluePoints);
        }

        [TestMethod]
        public void DetonableBombTest()
        {
            lowerLayer[4, 3] = LowerFieldType.DetonatorEffect;
            GameModel model = new GameModel(lowerLayer, upperLayer, 1, 100);
            model.FieldChanged += TesterHandler;
            model.NewCompetitionStarts(2, 1);
            model.Move(PlayerType.Red, Direction.Left);
            model.PlaceBomb(PlayerType.Red);
            Assert.AreEqual(2, counter);
            Thread.Sleep(6000);
            Assert.AreEqual(2, counter);
            model.PlaceBomb(PlayerType.Red);
            Assert.AreEqual(3, counter);
        }

        [TestMethod]
        [DataRow(LowerFieldType.InvincibilityEffect)]
        [DataRow(LowerFieldType.GhostEffect)]
        public void SpecialPowerTest(LowerFieldType lower)
        {
            lowerLayer[4, 3] = lower;
            GameModel model = new GameModel(lowerLayer, upperLayer, 1, 100);
            model.PlayerStatusChanged += TesterHandler;
            model.NewCompetitionStarts(2, 1);
            Assert.AreEqual(0, counter);
            model.Move(PlayerType.Red, Direction.Left);
            Assert.AreEqual(1, counter);
            Thread.Sleep(10100);
            Assert.AreEqual(8, counter);
        }

        [TestMethod]
        public void BattleRoyaleTest()
        {
            lowerLayer[1, 1] = LowerFieldType.InnerWall;
            lowerLayer[1, 2] = LowerFieldType.Box;
            lowerLayer[1, 3] = LowerFieldType.MoreBombsEffect;
            lowerLayer[1, 4] = LowerFieldType.Debuff;
            lowerLayer[2, 1] = LowerFieldType.BarrierEffect;
            lowerLayer[4, 5] = LowerFieldType.BarrierEffect;
            GameModel model = new GameModel(lowerLayer, upperLayer, 1, 7);
            model.NewCompetitionStarts(2, 1);
            model.FieldChanged += TesterHandler;
            model.Move(PlayerType.Blue, Direction.Left); // 2,1
            model.Move(PlayerType.Red, Direction.Right); // 4,5
            model.PlaceBarrier(PlayerType.Blue);
            model.PlaceBarrier(PlayerType.Red);
            Thread.Sleep(5000);
            model.Move(PlayerType.Blue, Direction.Down); // 3,1
            model.Move(PlayerType.Red, Direction.Up); // 3,5
            model.PlaceBomb(PlayerType.Blue);
            model.PlaceBomb(PlayerType.Red);
            Thread.Sleep(600);
            model.Move(PlayerType.Blue, Direction.Right); // 3,2
            model.Move(PlayerType.Red, Direction.Left); // 3,4
            counter = 0;
            Thread.Sleep(3000);
            Assert.AreEqual(16, counter);
            model.PlaceBomb(PlayerType.Blue);
            model.PlaceBomb(PlayerType.Red);
            Assert.AreEqual(18, counter);
        }
    }
}
