﻿namespace Test
{
    [TestClass]
    public class MonsterTest
    {
        #region Fields

        private readonly static int ghostInterval = 150;
        private readonly static int smartInterval = 100;
        private readonly static int basicInterval = 120;
        private readonly static int clumsyInterval = 120;
        private readonly static int delta = 50;

        private LowerFieldType[,] _lowerLayer = null!;
        private UpperFieldType[,] _upperLayer = null!;
        Func<int, int, (LowerFieldType, UpperFieldType)> _indexerFunction = null!;
        int counter;

        #endregion

        #region Event Handlers

        private void TesterHandlerDontStep(object? sender, MoveEventArgs e)
        {
            Assert.IsTrue(sender is Monster);
            counter++;
        }

        private void TesterHandlerDoStep(object? sender, MoveEventArgs e)
        {
            Assert.IsTrue(sender is Monster);
            if (sender is Monster monster)
            {
                monster.SuccessfulMove(e.ToX, e.ToY, e.Direction);
                counter++;
            }
        }

        #endregion

        #region TestInitialize

        [TestInitialize]
        public void TestInitialize()
        {
            counter = 0;
            _lowerLayer = new LowerFieldType[5, 5]
            {
                { LowerFieldType.OuterWall, LowerFieldType.OuterWall, LowerFieldType.OuterWall, LowerFieldType.OuterWall, LowerFieldType.OuterWall},
                { LowerFieldType.OuterWall, LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.OuterWall},
                { LowerFieldType.OuterWall, LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.OuterWall},
                { LowerFieldType.OuterWall, LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.OuterWall},
                { LowerFieldType.OuterWall, LowerFieldType.OuterWall, LowerFieldType.OuterWall, LowerFieldType.OuterWall, LowerFieldType.OuterWall}
            };
            _upperLayer = new UpperFieldType[5, 5]
            {
                { UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty},
                { UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty},
                { UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty},
                { UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty},
                { UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty}
            };
            _indexerFunction = (x, y) => (_lowerLayer[y, x], _upperLayer[y, x]);
        }

        #endregion

        #region ConstructorTest

        [TestMethod]
        [DataRow(1, 2)]
        [DataRow(3, 4)]
        public void ConstructorTest(int x, int y)
        {
            Func<int, int, (LowerFieldType, UpperFieldType)> pseudoIndexer = (x, y) => (LowerFieldType.Empty, UpperFieldType.Empty);
            Monster monster = new BasicMonster(x, y, pseudoIndexer);
            Assert.AreEqual(x, monster.X);
            Assert.AreEqual(y, monster.Y);

            monster = new ClumsyMonster(x, y, pseudoIndexer);
            Assert.AreEqual(x, monster.X);
            Assert.AreEqual(y, monster.Y);

            monster = new GhostMonster(x, y, pseudoIndexer);
            Assert.AreEqual(x, monster.X);
            Assert.AreEqual(y, monster.Y);

            monster = new SmartMonster(x, y, pseudoIndexer);
            Assert.AreEqual(x, monster.X);
            Assert.AreEqual(y, monster.Y);
        }

        #endregion

        #region WantsToMoveTests

        [TestMethod]
        public void BasicMonsterMonsterWantsToMove()
        {
            Func<int, int, (LowerFieldType, UpperFieldType)> pseudoIndexer = (x, y) => (LowerFieldType.Empty, UpperFieldType.Empty); ;

            BasicMonster m = new(1, 1, pseudoIndexer, basicInterval);
            m.MonsterWantsToMove += TesterHandlerDontStep;
            Thread.Sleep(delta);
            Assert.AreEqual(0, counter);
            Thread.Sleep(basicInterval);
            Assert.AreEqual(1, counter);
        }

        [TestMethod]
        public void SmartMonsterWantsToMove()
        {
            Func<int, int, (LowerFieldType, UpperFieldType)> pseudoIndexer = (x, y) => (LowerFieldType.Empty, UpperFieldType.Empty); ;

            SmartMonster m = new(1, 1, pseudoIndexer, smartInterval);
            m.MonsterWantsToMove += TesterHandlerDontStep;
            Thread.Sleep(delta);
            Assert.AreEqual(0, counter);
            Thread.Sleep(smartInterval);
            Assert.AreEqual(1, counter);
        }

        [TestMethod]
        public void ClumsyMonsterWantsToMove()
        {
            Func<int, int, (LowerFieldType, UpperFieldType)> pseudoIndexer = (x, y) => (LowerFieldType.Empty, UpperFieldType.Empty); ;

            ClumsyMonster m = new(1, 1, pseudoIndexer, clumsyInterval);
            m.MonsterWantsToMove += TesterHandlerDontStep;
            Thread.Sleep(delta);
            Assert.AreEqual(0, counter);
            Thread.Sleep(clumsyInterval);
            Assert.AreEqual(1, counter);
        }

        [TestMethod]
        public void GhostMonsterWantsToMove()
        {
            Func<int, int, (LowerFieldType, UpperFieldType)> pseudoIndexer = (x, y) => (LowerFieldType.Empty, UpperFieldType.Empty); ;

            GhostMonster m = new(1, 1, pseudoIndexer, ghostInterval);
            m.MonsterWantsToMove += TesterHandlerDontStep;
            Thread.Sleep(delta);
            Assert.AreEqual(0, counter);
            Thread.Sleep(ghostInterval);
            Assert.AreEqual(1, counter);
        }

        #endregion

        #region NoAvailablePositionTests

        [TestMethod]
        [DataRow(LowerFieldType.InnerWall)]
        [DataRow(LowerFieldType.Box)]
        [DataRow(LowerFieldType.BlueBarrier)]
        [DataRow(LowerFieldType.RedBarrier)]
        [DataRow(LowerFieldType.Bomb)]
        public void BasicNoAvailablePosition(LowerFieldType lower)
        {
            _upperLayer[1, 1] = UpperFieldType.BasicMonster;
            _lowerLayer[2, 1] = lower;
            _lowerLayer[1, 2] = lower;

            BasicMonster m = new(1, 1, _indexerFunction, basicInterval);
            m.MonsterWantsToMove += TesterHandlerDoStep;
            Thread.Sleep(basicInterval + delta);
            Assert.AreEqual(0, counter);
            Assert.AreEqual(1, m.X);
            Assert.AreEqual(1, m.Y);
        }

        [TestMethod]
        [DataRow(LowerFieldType.InnerWall)]
        [DataRow(LowerFieldType.Box)]
        [DataRow(LowerFieldType.BlueBarrier)]
        [DataRow(LowerFieldType.RedBarrier)]
        [DataRow(LowerFieldType.Bomb)]
        public void ClumsyNoAvailablePosition(LowerFieldType lower)
        {
            _upperLayer[1, 1] = UpperFieldType.BasicMonster;
            _lowerLayer[2, 1] = lower;
            _lowerLayer[1, 2] = lower;

            ClumsyMonster m = new(1, 1, _indexerFunction, clumsyInterval);
            m.MonsterWantsToMove += TesterHandlerDoStep;
            Thread.Sleep(clumsyInterval + delta);
            Assert.AreEqual(0, counter);
            Assert.AreEqual(1, m.X);
            Assert.AreEqual(1, m.Y);
        }


        [TestMethod]
        [DataRow(LowerFieldType.InnerWall)]
        [DataRow(LowerFieldType.Box)]
        [DataRow(LowerFieldType.BlueBarrier)]
        [DataRow(LowerFieldType.RedBarrier)]
        [DataRow(LowerFieldType.Bomb)]
        public void SmartNoAvailablePosition(LowerFieldType lower)
        {
            _upperLayer[1, 1] = UpperFieldType.BasicMonster;
            _lowerLayer[2, 1] = lower;
            _lowerLayer[1, 2] = lower;

            SmartMonster m = new(1, 1, _indexerFunction, smartInterval);
            m.MonsterWantsToMove += TesterHandlerDoStep;
            Thread.Sleep(smartInterval + delta);
            Assert.AreEqual(0, counter);
            Assert.AreEqual(1, m.X);
            Assert.AreEqual(1, m.Y);
        }


        [TestMethod]
        public void GhostNoAvailablePosition()
        {
            _upperLayer[1, 1] = UpperFieldType.BasicMonster;
            _lowerLayer[2, 1] = LowerFieldType.OuterWall;
            _lowerLayer[1, 2] = LowerFieldType.OuterWall;

            GhostMonster m = new(1, 1, _indexerFunction, ghostInterval);
            m.MonsterWantsToMove += TesterHandlerDoStep;
            Thread.Sleep(ghostInterval + delta);
            Assert.AreEqual(0, counter);
            Assert.AreEqual(1, m.X);
            Assert.AreEqual(1, m.Y);
        }

        #endregion

        #region OneAvailablePositionTests

        [TestMethod]
        [DataRow(LowerFieldType.Empty)]
        [DataRow(LowerFieldType.MoreBombsEffect)]
        [DataRow(LowerFieldType.Debuff)]
        public void BasicOneAvailablePosition(LowerFieldType lower)
        {
            _upperLayer[1, 1] = UpperFieldType.BasicMonster;
            _lowerLayer[2, 1] = LowerFieldType.OuterWall;
            _lowerLayer[1, 2] = lower;

            BasicMonster m = new(1, 1, _indexerFunction, basicInterval);
            m.MonsterWantsToMove += TesterHandlerDoStep;
            Thread.Sleep(basicInterval + delta);
            Assert.AreEqual(1, counter);
            Assert.AreEqual(2, m.X);
            Assert.AreEqual(1, m.Y);
        }

        [TestMethod]
        [DataRow(LowerFieldType.Empty)]
        [DataRow(LowerFieldType.MoreBombsEffect)]
        [DataRow(LowerFieldType.Debuff)]
        public void ClumsyOneAvailablePosition(LowerFieldType lower)
        {
            _upperLayer[1, 1] = UpperFieldType.BasicMonster;
            _lowerLayer[2, 1] = LowerFieldType.OuterWall;
            _lowerLayer[1, 2] = lower;

            ClumsyMonster m = new(1, 1, _indexerFunction, clumsyInterval);
            m.MonsterWantsToMove += TesterHandlerDoStep;
            Thread.Sleep(clumsyInterval + delta);
            Assert.AreEqual(1, counter);
            Assert.AreEqual(2, m.X);
            Assert.AreEqual(1, m.Y);
        }

        [TestMethod]
        [DataRow(LowerFieldType.Empty)]
        [DataRow(LowerFieldType.MoreBombsEffect)]
        [DataRow(LowerFieldType.Debuff)]
        public void SmartOneAvailablePosition(LowerFieldType lower)
        {
            _upperLayer[1, 1] = UpperFieldType.BasicMonster;
            _lowerLayer[2, 1] = LowerFieldType.OuterWall;
            _lowerLayer[1, 2] = lower;

            SmartMonster m = new(1, 1, _indexerFunction, smartInterval);
            m.MonsterWantsToMove += TesterHandlerDoStep;
            Thread.Sleep(smartInterval + delta);
            Assert.AreEqual(1, counter);
            Assert.AreEqual(2, m.X);
            Assert.AreEqual(1, m.Y);
        }

        [TestMethod]
        [DataRow(LowerFieldType.Empty)]
        [DataRow(LowerFieldType.MoreBombsEffect)]
        [DataRow(LowerFieldType.Debuff)]
        [DataRow(LowerFieldType.RedBarrier)]
        [DataRow(LowerFieldType.BlueBarrier)]
        [DataRow(LowerFieldType.InnerWall)]
        [DataRow(LowerFieldType.Box)]
        public void GhostOneAvailablePosition(LowerFieldType lower)
        {
            _upperLayer[1, 1] = UpperFieldType.BasicMonster;
            _lowerLayer[2, 1] = LowerFieldType.OuterWall;
            _lowerLayer[1, 2] = lower;

            GhostMonster m = new(1, 1, _indexerFunction, ghostInterval);
            m.MonsterWantsToMove += TesterHandlerDoStep;
            Thread.Sleep(ghostInterval + delta);
            Assert.AreEqual(1, counter);
            Assert.AreEqual(2, m.X);
            Assert.AreEqual(1, m.Y);
        }

        #endregion

        #region SmartMonsterBFSTests

        [TestMethod]
        public void SmartMonsterBFSTest1()
        {
            _upperLayer[1, 1] = UpperFieldType.SmartMonster;
            _lowerLayer[1, 2] = LowerFieldType.InnerWall; // if we put an innerwall here, we will for sure that the monster will go down (initial direction is random!)
            _upperLayer[3, 3] = UpperFieldType.BluePlayer;
            _upperLayer[1, 3] = UpperFieldType.RedPlayer;

            SmartMonster monster = new(1, 1, _indexerFunction, smartInterval);
            monster.MonsterWantsToMove += TesterHandlerDoStep;

            Thread.Sleep(delta); // doesn't move yet
            Assert.AreEqual(0, counter);
            Assert.AreEqual(1, monster.X);
            Assert.AreEqual(1, monster.Y);

            Thread.Sleep(smartInterval); // moves down, only one way
            Assert.AreEqual(1, counter);
            Assert.AreEqual(1, monster.X);
            Assert.AreEqual(2, monster.Y);

            Thread.Sleep(smartInterval); // moves down again, because doesn't change direction
            Assert.AreEqual(2, counter);
            Assert.AreEqual(1, monster.X);
            Assert.AreEqual(3, monster.Y);

            Thread.Sleep(smartInterval); // two ways, but always go to the right because blue is closer
            Assert.AreEqual(3, counter);
            Assert.AreEqual(2, monster.X);
            Assert.AreEqual(3, monster.Y);
        }

        [TestMethod]
        public void SmartMonsterBFSTest2()
        {
            _upperLayer[3, 3] = UpperFieldType.SmartMonster;
            _lowerLayer[3, 2] = LowerFieldType.InnerWall; // if we put an innerwall here, we will for sure that the monster will go down (initial direction is random!)
            _upperLayer[1, 1] = UpperFieldType.BluePlayer;
            _upperLayer[3, 1] = UpperFieldType.RedPlayer;

            SmartMonster monster = new(3, 3, _indexerFunction, smartInterval);
            monster.MonsterWantsToMove += TesterHandlerDoStep;

            Thread.Sleep(delta); // doesn't move yet
            Assert.AreEqual(0, counter);
            Assert.AreEqual(3, monster.X);
            Assert.AreEqual(3, monster.Y);

            Thread.Sleep(smartInterval); // moves down, only one way
            Assert.AreEqual(1, counter);
            Assert.AreEqual(3, monster.X);
            Assert.AreEqual(2, monster.Y);

            Thread.Sleep(smartInterval); // moves down again, because doesn't change direction
            Assert.AreEqual(2, counter);
            Assert.AreEqual(3, monster.X);
            Assert.AreEqual(1, monster.Y);

            Thread.Sleep(smartInterval); // two ways, but always go to the right because blue is closer
            Assert.AreEqual(3, counter);
            Assert.AreEqual(2, monster.X);
            Assert.AreEqual(1, monster.Y);
        }

        #endregion

        #region SeveralAvailablePositionTests

        [TestMethod]
        [DataRow(LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.Box)]
        [DataRow(LowerFieldType.Debuff, LowerFieldType.Empty, LowerFieldType.BlueBarrier)]
        [DataRow(LowerFieldType.Empty, LowerFieldType.MoreBombsEffect, LowerFieldType.InnerWall)]
        [DataRow(LowerFieldType.BiggerBombsEffect, LowerFieldType.Empty, LowerFieldType.Bomb)]
        public void BasicSeveralAvailable(LowerFieldType canStep1, LowerFieldType canStep2, LowerFieldType cantStep)
        {
            _upperLayer[1, 2] = UpperFieldType.BasicMonster;
            _lowerLayer[1, 3] = canStep1;
            _lowerLayer[2, 2] = canStep2;
            _lowerLayer[1, 1] = cantStep;

            BasicMonster m = new(2, 1, _indexerFunction, basicInterval);
            m.MonsterWantsToMove += TesterHandlerDoStep;
            Thread.Sleep(basicInterval + delta);
            Assert.AreEqual(1, counter);
            Assert.IsTrue((m.X == 3 && m.Y == 1) || (m.X == 2 && m.Y == 2)); //  we don't set the seed, we just check if the move was valid
        }

        [TestMethod]
        [DataRow(LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.OuterWall)]
        [DataRow(LowerFieldType.Debuff, LowerFieldType.InnerWall, LowerFieldType.OuterWall)]
        [DataRow(LowerFieldType.InnerWall, LowerFieldType.InnerWall, LowerFieldType.OuterWall)]
        [DataRow(LowerFieldType.InnerWall, LowerFieldType.Empty, LowerFieldType.OuterWall)]
        public void GhostSeveralAvailable(LowerFieldType canStep1, LowerFieldType canStep2, LowerFieldType cantStep)
        {
            _upperLayer[1, 2] = UpperFieldType.GhostMonster;
            _lowerLayer[1, 3] = canStep1;
            _lowerLayer[2, 2] = canStep2;
            _lowerLayer[1, 1] = cantStep;

            GhostMonster m = new(2, 1, _indexerFunction, ghostInterval);
            m.MonsterWantsToMove += TesterHandlerDoStep;
            Thread.Sleep(ghostInterval + delta);
            Assert.AreEqual(1, counter);
            Assert.IsTrue((m.X == 3 && m.Y == 1) || (m.X == 2 && m.Y == 2)); //  we don't set the seed, we just check if the move was valid
        }

        [TestMethod]
        [DataRow(LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.Box)]
        [DataRow(LowerFieldType.Debuff, LowerFieldType.Empty, LowerFieldType.BlueBarrier)]
        [DataRow(LowerFieldType.Empty, LowerFieldType.MoreBombsEffect, LowerFieldType.InnerWall)]
        [DataRow(LowerFieldType.BiggerBombsEffect, LowerFieldType.Empty, LowerFieldType.Bomb)]
        public void ClumsySeveralAvailable(LowerFieldType canStep1, LowerFieldType canStep2, LowerFieldType cantStep)
        {
            _upperLayer[1, 2] = UpperFieldType.ClumsyMonster;
            _lowerLayer[1, 3] = canStep1;
            _lowerLayer[2, 2] = canStep2;
            _lowerLayer[1, 1] = cantStep;

            ClumsyMonster m = new(2, 1, _indexerFunction, clumsyInterval);
            m.MonsterWantsToMove += TesterHandlerDoStep;
            Thread.Sleep(clumsyInterval + delta);
            Assert.AreEqual(1, counter);
            Assert.IsTrue((m.X == 3 && m.Y == 1) || (m.X == 2 && m.Y == 2)); //  we don't set the seed, we just check if the move was valid
        }

        [TestMethod]
        [DataRow(LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.Box)]
        [DataRow(LowerFieldType.Debuff, LowerFieldType.Empty, LowerFieldType.BlueBarrier)]
        [DataRow(LowerFieldType.Empty, LowerFieldType.MoreBombsEffect, LowerFieldType.InnerWall)]
        [DataRow(LowerFieldType.BiggerBombsEffect, LowerFieldType.Empty, LowerFieldType.Bomb)]
        public void SmartSeveralAvailable(LowerFieldType canStep1, LowerFieldType canStep2, LowerFieldType cantStep)
        {
            _upperLayer[1, 2] = UpperFieldType.SmartMonster;
            _lowerLayer[1, 3] = canStep1;
            _lowerLayer[2, 2] = canStep2;
            _lowerLayer[1, 1] = cantStep;

            SmartMonster m = new(2, 1, _indexerFunction, smartInterval);
            m.MonsterWantsToMove += TesterHandlerDoStep;
            Thread.Sleep(smartInterval + delta);
            Assert.AreEqual(1, counter);
            Assert.IsTrue((m.X == 3 && m.Y == 1) || (m.X == 2 && m.Y == 2)); //  we don't set the seed, we just check if the move was valid
        }

        #endregion
    }
}
