﻿namespace Test
{
    [TestClass]
    public class PlayerTest
    {
        #region Fields

        int counter;

        private LowerFieldType[,] _lowerLayer = null!;
        private UpperFieldType[,] _upperLayer = null!;
        Func<int, int, (LowerFieldType, UpperFieldType)> _indexerFunction = null!;

        private readonly static int interval = 100;
        private readonly static int delta = 30;

        #endregion

        #region Event Handlers

        private void TesterHandler(object? sender, EventArgs e)
        {
            ++counter;
            Assert.IsTrue(sender is Player);
        }

        #endregion

        #region TestInitialize

        [TestInitialize]
        public void TestInitialize()
        {
            counter = 0;
            _lowerLayer = new LowerFieldType[5, 5]
            {
                { LowerFieldType.OuterWall, LowerFieldType.OuterWall, LowerFieldType.OuterWall, LowerFieldType.OuterWall, LowerFieldType.OuterWall},
                { LowerFieldType.OuterWall, LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.OuterWall},
                { LowerFieldType.OuterWall, LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.OuterWall},
                { LowerFieldType.OuterWall, LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.Empty, LowerFieldType.OuterWall},
                { LowerFieldType.OuterWall, LowerFieldType.OuterWall, LowerFieldType.OuterWall, LowerFieldType.OuterWall, LowerFieldType.OuterWall}
            };
            _upperLayer = new UpperFieldType[5, 5]
            {
                { UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty},
                { UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty},
                { UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty},
                { UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty},
                { UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty, UpperFieldType.Empty}
            };
            _indexerFunction = (x, y) => (_lowerLayer[y, x], _upperLayer[y, x]);
        }

        #endregion

        #region ConstructorTest

        [TestMethod]
        [DataRow(1, 2)]
        [DataRow(3, 4)]
        public void ConstructorTest(int x, int y)
        {
            Func<int, int, (LowerFieldType, UpperFieldType)> pseudoIndexer = (x, y) => (LowerFieldType.Empty, UpperFieldType.Empty);
            Player player = new(x, y, pseudoIndexer);
            Assert.AreEqual(x, player.X);
            Assert.AreEqual(y, player.Y);
        }

        #endregion

        [TestMethod]
        [DataRow(LowerFieldType.InnerWall)]
        [DataRow(LowerFieldType.Box)]
        [DataRow(LowerFieldType.BlueBarrier)]
        [DataRow(LowerFieldType.RedBarrier)]
        [DataRow(LowerFieldType.Bomb)]
        public void BasicPlayerMovesTest(LowerFieldType lower)
        {
            Player player = new(1, 1, _indexerFunction, interval);
            _lowerLayer[1, 2] = lower;

            Assert.IsFalse(player.PlayerMoves(Direction.Left));
            Assert.IsFalse(player.PlayerMoves(Direction.Up));
            Assert.IsFalse(player.PlayerMoves(Direction.Right));
            Assert.IsTrue(player.PlayerMoves(Direction.Down));
            Thread.Sleep(interval * 2);
            Assert.IsTrue(player.PlayerMoves(Direction.Down));
            Thread.Sleep(interval / 2);
            Assert.IsFalse(player.PlayerMoves(Direction.Right));
        }

        [TestMethod]
        [DataRow(LowerFieldType.InnerWall)]
        [DataRow(LowerFieldType.Box)]
        [DataRow(LowerFieldType.BlueBarrier)]
        [DataRow(LowerFieldType.RedBarrier)]
        [DataRow(LowerFieldType.Bomb)]
        public void GhostPlayerMovesTest(LowerFieldType lower)
        {
            Player player = new(1, 1, _indexerFunction, interval);
            _lowerLayer[1, 2] = LowerFieldType.GhostEffect;
            player.PlayerMoves(Direction.Right);
            Assert.IsTrue(player.IsGhost);
            Thread.Sleep(interval * 2);

            _lowerLayer[1, 3] = lower;

            Assert.IsTrue(player.PlayerMoves(Direction.Right));
            Thread.Sleep(interval * 2);
            Assert.IsFalse(player.PlayerMoves(Direction.Up));
            Assert.IsFalse(player.PlayerMoves(Direction.Right));
            Assert.IsTrue(player.PlayerMoves(Direction.Down));
            Thread.Sleep(interval * 2);
            Assert.IsTrue(player.PlayerMoves(Direction.Down));
            Thread.Sleep(interval / 2);
            Assert.IsFalse(player.PlayerMoves(Direction.Right));
        }

        [TestMethod]
        [DataRow(LowerFieldType.InnerWall)]
        [DataRow(LowerFieldType.Box)]
        [DataRow(LowerFieldType.BlueBarrier)]
        [DataRow(LowerFieldType.RedBarrier)]
        [DataRow(LowerFieldType.Bomb)]
        public void InvinciblePlayerMovesTest(LowerFieldType lower)
        {
            Player player = new(1, 1, _indexerFunction, interval);
            _lowerLayer[1, 2] = LowerFieldType.InvincibilityEffect;
            player.PlayerMoves(Direction.Right);
            Assert.IsTrue(player.IsInvincible);
            _lowerLayer[1, 3] = lower;
            player.PlayerMoves(Direction.Right);

            Assert.IsFalse(player.PlayerMoves(Direction.Up));
            Assert.IsFalse(player.PlayerMoves(Direction.Right));
            Thread.Sleep(interval * 2);
            Assert.IsTrue(player.PlayerMoves(Direction.Down));
            Thread.Sleep(interval * 2);
            Assert.IsTrue(player.PlayerMoves(Direction.Down));
            Thread.Sleep(interval / 2);
            Assert.IsFalse(player.PlayerMoves(Direction.Right));
        }

        [TestMethod]
        [DataRow(LowerFieldType.InnerWall)]
        [DataRow(LowerFieldType.Box)]
        [DataRow(LowerFieldType.BlueBarrier)]
        [DataRow(LowerFieldType.RedBarrier)]
        [DataRow(LowerFieldType.Bomb)]
        public void InvincibleGhostPlayerMovesTest(LowerFieldType lower)
        {
            Player player = new(1, 1, _indexerFunction, interval);
            _lowerLayer[1, 2] = LowerFieldType.InvincibilityEffect;
            player.PlayerMoves(Direction.Right);
            Assert.IsTrue(player.IsInvincible);
            _lowerLayer[1, 3] = LowerFieldType.GhostEffect;
            Thread.Sleep(interval * 2);
            player.PlayerMoves(Direction.Right);
            Assert.IsTrue(player.IsGhost);

            Thread.Sleep(interval * 2);
            Assert.IsFalse(player.PlayerMoves(Direction.Up));
            Thread.Sleep(interval * 2);
            Assert.IsFalse(player.PlayerMoves(Direction.Right));
            _lowerLayer[2, 3] = lower;
            Assert.IsTrue(player.PlayerMoves(Direction.Down));

            Thread.Sleep(interval * 2);
            Assert.IsTrue(player.PlayerMoves(Direction.Down));
            Thread.Sleep(interval * 2);

            Assert.IsTrue(player.PlayerMoves(Direction.Up));
            Thread.Sleep(interval * 2);
            Assert.IsTrue(player.PlayerMoves(Direction.Up));
            Thread.Sleep(interval * 2);
            Assert.IsTrue(player.PlayerMoves(Direction.Left));
        }

        [TestMethod]
        [DataRow(UpperFieldType.BasicMonster)]
        [DataRow(UpperFieldType.SmartMonster)]
        [DataRow(UpperFieldType.ClumsyMonster)]
        [DataRow(UpperFieldType.GhostMonster)]
        public void InvinciblePlayerDiesTest(UpperFieldType upper)
        {
            Player player = new(1, 1, _indexerFunction, interval);
            player.PlayerDied += TesterHandler;
            _lowerLayer[1, 2] = LowerFieldType.InvincibilityEffect;
            player.PlayerMoves(Direction.Right);
            Assert.IsTrue(player.IsInvincible);
            Assert.AreEqual(0, counter);
            _upperLayer[1, 3] = upper;
            player.PlayerMoves(Direction.Right);
            Assert.AreEqual(0, counter);
        }

        [TestMethod]
        [DataRow(UpperFieldType.BasicMonster)]
        [DataRow(UpperFieldType.SmartMonster)]
        [DataRow(UpperFieldType.ClumsyMonster)]
        [DataRow(UpperFieldType.GhostMonster)]
        public void PlayerDiesTest(UpperFieldType upper)
        {
            Player player = new(1, 1, _indexerFunction, interval);
            player.PlayerDied += TesterHandler;
            Assert.AreEqual(0, counter);
            _upperLayer[1, 2] = upper;
            player.PlayerMoves(Direction.Right);
            Assert.AreEqual(1, counter);
            Assert.IsTrue(player.IsDead);
            Thread.Sleep(interval * 2);
            Assert.IsFalse(player.PlayerMoves(Direction.Down));

            counter = 0;
            player = new(2, 2, _indexerFunction, interval);
            player.PlayerDied += TesterHandler;
            Assert.AreEqual(0, counter);
            _upperLayer[1, 2] = upper;
            player.PlayerMoves(Direction.Up);
            Assert.AreEqual(1, counter);

            counter = 0;
            player = new(2, 2, _indexerFunction, interval);
            player.PlayerDied += TesterHandler;
            Assert.AreEqual(0, counter);
            _upperLayer[2, 1] = upper;
            player.PlayerMoves(Direction.Left);
            Assert.AreEqual(1, counter);

            counter = 0;
            player = new(2, 2, _indexerFunction, interval);
            player.PlayerDied += TesterHandler;
            Assert.AreEqual(0, counter);
            _upperLayer[3, 2] = upper;
            player.PlayerMoves(Direction.Down);
            Assert.AreEqual(1, counter);
        }

        [TestMethod]
        public void ApplyMoreBombsBuffTest()
        {
            Player player = new(1, 1, _indexerFunction, interval);
            _lowerLayer[1, 2] = LowerFieldType.MoreBombsEffect;
            int maxBombCount = player.MaxBombCount;
            player.PlayerMoves(Direction.Right);
            Assert.AreEqual(maxBombCount + 1, player.MaxBombCount);
        }

        [TestMethod]
        public void ApplyBiggerBombsBuffTest()
        {
            Player player = new(1, 1, _indexerFunction, interval);
            _lowerLayer[1, 2] = LowerFieldType.BiggerBombsEffect;
            int maxBombRange = player.MaxBombRange;
            player.PlayerMoves(Direction.Right);
            Assert.AreEqual(maxBombRange + 1, player.MaxBombRange);
        }

        [TestMethod]
        public void ApplyDetonatorBuffTest()
        {
            Player player = new(1, 1, _indexerFunction, interval);
            _lowerLayer[1, 2] = LowerFieldType.DetonatorEffect;
            Assert.IsFalse(player.HasDetonator);
            player.PlayerMoves(Direction.Right);
            Assert.IsTrue(player.HasDetonator);
        }

        [TestMethod]
        public void ApplySkatesBuffTest()
        {
            Player player = new(1, 1, _indexerFunction, interval);
            _lowerLayer[1, 2] = LowerFieldType.SkatesEffect;
            Assert.IsFalse(player.HasSkates);
            double moveInt = player.MoveInterval;
            player.PlayerMoves(Direction.Right);
            Assert.IsTrue(player.HasSkates);
            Assert.AreEqual(moveInt / 2, player.MoveInterval);

            _lowerLayer[1, 3] = LowerFieldType.SkatesEffect;
            player.PlayerMoves(Direction.Right);
            Assert.IsTrue(player.HasSkates);
            Assert.AreEqual(moveInt / 2, player.MoveInterval);
        }

        [TestMethod]
        public void ApplyInvincibilityBuffTest()
        {
            Player player = new(1, 1, _indexerFunction, interval);
            player.StatusChanged += TesterHandler;
            player.InvincibilityBuffIsOver += TesterHandler;
            Assert.AreEqual(0, counter);

            _lowerLayer[1, 2] = LowerFieldType.InvincibilityEffect;
            Assert.IsFalse(player.IsInvincible);
            player.PlayerMoves(Direction.Right);
            Assert.IsTrue(player.IsInvincible);

            Thread.Sleep(10 * interval + delta);
            Assert.IsFalse(player.IsInvincible);
            Assert.AreEqual(9, counter); //1(Apply) + 7(Tick) + 1(Over)

            _lowerLayer[1, 3] = LowerFieldType.InvincibilityEffect;
            _lowerLayer[2, 3] = LowerFieldType.InvincibilityEffect;
            player.PlayerMoves(Direction.Right);
            Assert.IsTrue(player.IsInvincible);
            Thread.Sleep(4 * interval);
            Assert.IsTrue(player.IsInvincible);
            player.PlayerMoves(Direction.Down);
            Assert.IsTrue(player.IsInvincible);
            Thread.Sleep(8 * interval);
            Assert.IsTrue(player.IsInvincible);
            Thread.Sleep(2 * interval + delta);
            Assert.IsFalse(player.IsInvincible);
        }

        [TestMethod]
        public void ApplyGhostBuffTest()
        {
            Player player = new(1, 1, _indexerFunction, interval);
            player.StatusChanged += TesterHandler;
            player.GhostBuffIsOver += TesterHandler;
            Assert.AreEqual(0, counter);

            _lowerLayer[1, 2] = LowerFieldType.GhostEffect;
            Assert.IsFalse(player.IsGhost);
            player.PlayerMoves(Direction.Right);
            Assert.IsTrue(player.IsGhost);

            Thread.Sleep(10 * interval + 100);
            Assert.IsFalse(player.IsGhost);
            Assert.AreEqual(9, counter); //1(Apply) + 7(Tick) + 1(Over)

            _lowerLayer[1, 3] = LowerFieldType.GhostEffect;
            _lowerLayer[2, 3] = LowerFieldType.GhostEffect;
            player.PlayerMoves(Direction.Right);
            Assert.IsTrue(player.IsGhost);
            Thread.Sleep(4 * interval);
            Assert.IsTrue(player.IsGhost);
            player.PlayerMoves(Direction.Down);
            Assert.IsTrue(player.IsGhost);
            Thread.Sleep(8 * interval);
            Assert.IsTrue(player.IsGhost);
            Thread.Sleep(2 * interval + 100);
            Assert.IsFalse(player.IsGhost);
        }

        [TestMethod]
        public void ApplyBarrierBuffTest()
        {
            Player player = new(1, 1, _indexerFunction, interval);
            _lowerLayer[1, 2] = LowerFieldType.BarrierEffect;
            int maxBarrierCount = player.MaxBarrierCount;
            player.PlayerMoves(Direction.Right);
            Assert.AreEqual(maxBarrierCount + 3, player.MaxBarrierCount);
        }

        [TestMethod]
        public void PlaceBarrierTest()
        {
            Player player = new(1, 1, _indexerFunction, interval);
            int placeBarrierCount = player.PlacedBarrierCount;
            Assert.IsFalse(player.JustPlacedBarrier);
            player.BarrierPlaced();
            Assert.AreEqual(placeBarrierCount + 1, player.PlacedBarrierCount);
            Assert.IsTrue(player.JustPlacedBarrier);
        }

        [TestMethod]
        public void BarrierDestroyedTest()
        {
            Player player = new(1, 1, _indexerFunction, interval);
            int placeBarrierCount = player.PlacedBarrierCount;
            Assert.IsFalse(player.JustPlacedBarrier);
            player.BarrierDestroyed();
            Assert.AreEqual(placeBarrierCount - 1, player.PlacedBarrierCount);
        }

        [TestMethod]
        public void ApplyDebuffTest()
        {
            for (int i = 0; i < 100; i++)
            {
                Player player = new(1, 1, _indexerFunction);
                _lowerLayer[1, 2] = LowerFieldType.Debuff;
                bool appliedDebuff = false;
                double speed = player.MoveInterval;
                Assert.IsFalse(player.HasSmallBombs);
                Assert.IsFalse(player.CantPlaceBombs);
                Assert.IsFalse(player.InstantBombs);
                player.PlayerMoves(Direction.Right);

                if (player.MoveInterval == speed * 1.2 || player.HasSmallBombs || player.CantPlaceBombs || player.InstantBombs)
                {
                    appliedDebuff = true;
                }
                Assert.IsTrue(appliedDebuff);
            }
        }

        [TestMethod]
        public void ApplySlowDebuffTest()
        {
            Player player = new(1, 1, _indexerFunction, interval);
            double speed = player.MoveInterval;

            player.ApplySlowDebuff();
            Assert.AreEqual(speed * 1.2, player.MoveInterval);
            Thread.Sleep(interval + delta);
            Assert.AreEqual(speed, player.MoveInterval);

            player.ApplySlowDebuff();
            Thread.Sleep(interval / 2);
            player.ApplySlowDebuff();
            Assert.AreEqual(speed * 1.2 * 1.2, player.MoveInterval);
            Thread.Sleep(interval / 2 + delta / 2);
            Assert.AreEqual(speed * 1.2, player.MoveInterval);
            Thread.Sleep(interval / 2 + delta);
            Assert.AreEqual(speed, player.MoveInterval);
        }

        [TestMethod]
        public void ApplySmallBombsDebuffTest()
        {
            Player player = new(1, 1, _indexerFunction, interval);
            Assert.IsFalse(player.HasSmallBombs);

            player.ApplySmallBombsDebuff();
            Assert.IsTrue(player.HasSmallBombs);

            Thread.Sleep(interval + delta);
            Assert.IsFalse(player.HasSmallBombs);

            player.ApplySmallBombsDebuff();
            Thread.Sleep(interval / 2);
            player.ApplySmallBombsDebuff();
            Assert.IsTrue(player.HasSmallBombs);
            Thread.Sleep(interval / 2 + delta);
            Assert.IsTrue(player.HasSmallBombs);
            Thread.Sleep(interval / 2 + delta);
            Assert.IsFalse(player.HasSmallBombs);
        }

        [TestMethod]
        public void ApplyCantPlaceBombsDebuffTest()
        {
            Player player = new(1, 1, _indexerFunction, interval);
            Assert.IsFalse(player.CantPlaceBombs);

            player.ApplyCantPlaceBombsDebuff();
            Assert.IsTrue(player.CantPlaceBombs);

            Thread.Sleep(interval + delta);
            Assert.IsFalse(player.CantPlaceBombs);

            player.ApplyCantPlaceBombsDebuff();
            Thread.Sleep(interval / 2);
            player.ApplyCantPlaceBombsDebuff();
            Assert.IsTrue(player.CantPlaceBombs);
            Thread.Sleep(interval / 2 + delta);
            Assert.IsTrue(player.CantPlaceBombs);
            Thread.Sleep(interval / 2 + delta);
            Assert.IsFalse(player.CantPlaceBombs);
        }

        [TestMethod]
        public void ApplyInstantBombsDebuffTest()
        {
            Player player = new(1, 1, _indexerFunction, interval);
            Assert.IsFalse(player.InstantBombs);

            player.ApplyInstantBombsDebuff();
            Assert.IsTrue(player.InstantBombs);

            Thread.Sleep(interval + delta);
            Assert.IsFalse(player.InstantBombs);

            player.ApplyInstantBombsDebuff();
            Thread.Sleep(interval / 2);
            player.ApplyInstantBombsDebuff();
            Assert.IsTrue(player.InstantBombs);
            Thread.Sleep(interval / 2 + delta);
            Assert.IsTrue(player.InstantBombs);
            Thread.Sleep(interval / 2 + delta);
            Assert.IsFalse(player.InstantBombs);
        }
    }
}
