﻿namespace Test
{
    [TestClass]
    public class UtilsTest
    {
        [TestMethod]
        public void TestActionEventArgs()
        {
            Action action = () => throw new NotImplementedException();
            ActionEventArgs e = new(action);
            Assert.AreEqual(action, e.Action);

            action = () => Thread.Sleep(1000);
            e = new(action);
            Assert.AreEqual(action, e.Action);
        }

        [TestMethod]
        [DataRow(1, 4, LowerFieldType.Bomb)]
        [DataRow(-3244, 42069, LowerFieldType.SkatesEffect)]
        [DataRow(0, 0, LowerFieldType.Box)]
        public void TestFieldChangedEventArgs(int x, int y, LowerFieldType lower)
        {
            FieldChangedEventArgs e = new(x, y, lower);
            Assert.AreEqual(x, e.X);
            Assert.AreEqual(y, e.Y);
            Assert.AreEqual(lower, e.Type);
        }

        [TestMethod]
        [DataRow(24)]
        [DataRow(-3244)]
        [DataRow(0)]
        public void TestIntEventArgs(int value)
        {
            IntEventArgs e = new(value);
            Assert.AreEqual(value, e.Value);
        }

        [TestMethod]
        [DataRow(1, 2, 3, 4, Direction.Up)]
        [DataRow(24, 24, 24, 25, Direction.Down)]
        [DataRow(-32, 248, 98, 78, Direction.Right)]
        public void TestMoveEventArgs(int fromX, int fromY, int toX, int toY, Direction direction)
        {
            MoveEventArgs e = new(fromX, fromY, toX, toY, direction);
            Assert.AreEqual(fromX, e.FromX);
            Assert.AreEqual(fromY, e.FromY);
            Assert.AreEqual(toX, e.ToX);
            Assert.AreEqual(toY, e.ToY);
            Assert.AreEqual(direction, e.Direction);
        }

        [TestMethod]
        public void TestNewRoundEventArgs()
        {
            LowerFieldType[,] lower = new LowerFieldType[1, 1] { { LowerFieldType.BarrierEffect } };
            UpperFieldType[,] upper = new UpperFieldType[2, 2] { { UpperFieldType.BasicMonster, UpperFieldType.BluePlayer },
                                                                 { UpperFieldType.RedPlayer,    UpperFieldType.Empty } };
            NewRoundEventArgs e = new(lower, upper);
            Assert.AreEqual(lower, e.LowerLayer);
            Assert.AreEqual(upper, e.UpperLayer);
        }

        [TestMethod]
        [DataRow(0)]
        [DataRow(1)]
        [DataRow(2)]
        public void TestPlayerEventArgs(PlayerType player)
        {
            PlayerEventArgs e = new(player);
            Assert.AreEqual(player, e.Player);
        }

        [TestMethod]
        [DataRow(SpecialPower.Invincibility, true, PlayerType.Blue)]
        [DataRow(SpecialPower.Ghost, true, PlayerType.Red)]
        [DataRow(SpecialPower.Invincibility, false, PlayerType.Red)]
        [DataRow(SpecialPower.Ghost, false, PlayerType.Blue)]
        public void TestPlayerStatusEventArgs(SpecialPower power, bool show, PlayerType player)
        {
            PlayerStatusEventArgs e = new(power, show, player);
            Assert.AreEqual(power, e.Power);
            Assert.AreEqual(show, e.Show);
            Assert.AreEqual(player, e.Player);
        }

        [TestMethod]
        [DataRow(24, 621)]
        [DataRow(-3244, 3)]
        [DataRow(0, 0)]
        public void TestPositionEventArgs(int x, int y)
        {
            PositionEventArgs e = new(x, y);
            Assert.AreEqual(x, e.X);
            Assert.AreEqual(y, e.Y);
        }

        [TestMethod]
        [DataRow(SpecialPower.Invincibility, true)]
        [DataRow(SpecialPower.Invincibility, false)]
        [DataRow(SpecialPower.Ghost, true)]
        [DataRow(SpecialPower.Ghost, false)]
        public void TestStatusEventArgs(SpecialPower power, bool show)
        {
            StatusEventArgs e = new(power, show);
            Assert.AreEqual(power, e.Power);
            Assert.AreEqual(show, e.Show);
        }
    }
}
