﻿using Model;
using Model.Utils;
using ViewModel;
using System.Windows;

namespace View
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application, IDisposable
    {
        private GameModel _model = null!;
        private MainViewModel _viewModel = null!;
        private MainWindow _mainWindow = null!;

        /// <summary>
        /// Initializes a new instance of the View.App class.
        /// </summary>
        public App()
        {
            Startup += App_OnStartup;
        }

        private void App_OnStartup(object? sender, StartupEventArgs args)
        {
            _model = new();
            _viewModel = new(_model);
            _viewModel.ExecuteOnMainThread += ViewModel_ExecuteOnMainThread;
            _viewModel.CompetitionOver += ViewModel_CompetitionOver;
            _viewModel.QuitApplication += ViewModel_QuitApplication;

            _mainWindow = new()
            {
                DataContext = _viewModel,
            };

            _mainWindow.Show();
        }

        private void ViewModel_ExecuteOnMainThread(object? sender, ActionEventArgs e)
        {
            Dispatcher.BeginInvoke(e.Action);
        }

        private void ViewModel_CompetitionOver(object? sender, PlayerEventArgs e)
        {
            if (e.Player == PlayerType.Blue)
            {
                MessageBox.Show(_mainWindow, "Congratulations! Blue Player Won!", "Game Over", MessageBoxButton.OK);
            }
            else
            {
                MessageBox.Show(_mainWindow, "Congratulations! Red Player Won!", "Game Over", MessageBoxButton.OK);
            }
        }

        private void ViewModel_QuitApplication(object? sender, EventArgs e)
        {
            _mainWindow.Close();
        }

        #region IDisposable

        private bool _disposed = false;

        ~App() => Dispose(false);

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!_disposed)
            {
                if (disposing)
                {
                    _model.Dispose();
                }

                _disposed = true;
            }
        }

        #endregion
    }
}
