﻿using System.Globalization;
using System.Windows.Data;
using System.Windows.Media.Imaging;

namespace View
{
    /// <summary>
    /// Converts boolean values to pictures that indicates if a power is active or not.
    /// </summary>
    internal class InvincibilityIconConverter : IValueConverter
    {
        private readonly static BitmapImage _coloredIcon = new(new Uri("Media\\shield.png",    UriKind.Relative));
        private readonly static BitmapImage _grayIcon    = new(new Uri("Media\\shield_bw.png", UriKind.Relative));

        /// <summary>
        /// Converts a boolean value to a picture, that represents if a power is active or not.
        /// </summary>
        /// <param name="value">The boolean value.</param>
        /// <param name="targetType">Ignored. It is expected to be a BitmapImage or one of it's bases.</param>
        /// <param name="parameter">Ignored.</param>
        /// <param name="culture">Ignored.</param>
        /// <returns>The BitmapImage that corresponds to the input value.</returns>
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (value is bool isInvincible && isInvincible)
                return _coloredIcon;
            return _grayIcon;
        }

        /// <summary>
        /// Not implemented. Throws NotImplementedException.
        /// </summary>
        /// <exception cref="NotImplementedException">Thrown every time.</exception>
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
