﻿using ViewModel;
using System.Windows;
using System.Windows.Input;

namespace View
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// Initializes a new instance of the View.MainWindow class.
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            KeyDown += OnKeyDown;
            KeyUp += OnKeyUp;
        }

        private void OnKeyDown(object sender, KeyEventArgs e)
        {
            if (DataContext is MainViewModel viewModel)
            {
                switch (e.Key)
                {
                    case Key.W: viewModel.BlueUpPressed      = true; break;
                    case Key.S: viewModel.BlueDownPressed    = true; break;
                    case Key.A: viewModel.BlueLeftPressed    = true; break;
                    case Key.D: viewModel.BlueRightPressed   = true; break;
                    case Key.Q: viewModel.BlueBombPressed    = true; break;
                    case Key.E: viewModel.BlueBarrierPressed = true; break;

                    case Key.I: viewModel.RedUpPressed       = true; break;
                    case Key.K: viewModel.RedDownPressed     = true; break;
                    case Key.J: viewModel.RedLeftPressed     = true; break;
                    case Key.L: viewModel.RedRightPressed    = true; break;
                    case Key.U: viewModel.RedBombPressed     = true; break;
                    case Key.O: viewModel.RedBarrierPressed  = true; break;
                }
                System.Diagnostics.Debug.WriteLine($"Key down: {e.Key}");
            }
            else
                throw new InvalidOperationException($"{nameof(DataContext)} was not an instance of {nameof(MainViewModel)}!");
        }

        private void OnKeyUp(object sender, KeyEventArgs e)
        {
            if (DataContext is MainViewModel viewModel)
            {
                switch (e.Key)
                {
                    case Key.W: viewModel.BlueUpPressed      = false; break;
                    case Key.S: viewModel.BlueDownPressed    = false; break;
                    case Key.A: viewModel.BlueLeftPressed    = false; break;
                    case Key.D: viewModel.BlueRightPressed   = false; break;
                    case Key.E: viewModel.BlueBarrierPressed = false; break;
                    case Key.Q:
                                viewModel.BlueBombPressed    = false;
                                viewModel.BlueBombRepeat     = false;
                                break;

                    case Key.I: viewModel.RedUpPressed       = false; break;
                    case Key.K: viewModel.RedDownPressed     = false; break;
                    case Key.J: viewModel.RedLeftPressed     = false; break;
                    case Key.L: viewModel.RedRightPressed    = false; break;
                    case Key.O: viewModel.RedBarrierPressed  = false; break;
                    case Key.U:
                                viewModel.RedBombPressed     = false;
                                viewModel.RedBombRepeat      = false;
                                break;
                }
                System.Diagnostics.Debug.WriteLine($"Key up: {e.Key}");
            }
            else
                throw new InvalidOperationException($"{nameof(DataContext)} was not an instance of {nameof(MainViewModel)}!");
        }
    }
}
