﻿using System.Windows;
using System.Windows.Controls;

namespace View
{
    /// <summary>
    /// Represents a button that can be in a selected or non-selected state.
    /// </summary>
    public class SelectableButton : Button
    {
        /// <summary>
        /// The dependency property that registered the IsSelected property.
        /// </summary>
        public static readonly DependencyProperty IsSelectedProperty = DependencyProperty.Register(
            name: "IsSelected",
            propertyType: typeof(bool),
            ownerType: typeof(SelectableButton),
            typeMetadata: new FrameworkPropertyMetadata(defaultValue: false)
        );

        /// <summary>
        /// Gets or sets the value that indicates if the button is selected.
        /// </summary>
        public bool IsSelected
        {
            get => (bool)GetValue(IsSelectedProperty);
            set => SetValue(IsSelectedProperty, value);
        }

        /// <summary>
        /// Initializes a new instance of the View.SelectableButton class.
        /// </summary>
        public SelectableButton() : base() { }
    }
}
