﻿using Model.Utils;

namespace ViewModel
{
    /// <summary>
    /// Represents a tile in the game.
    /// </summary>
    public class GameField : ViewModelBase
    {
        private LowerFieldType _lowerLayer;
        private UpperFieldType _upperLayer;
        private Direction _dir;

        /// <summary>
        /// Gets or sets the the value of the lower layer.
        /// </summary>
        public LowerFieldType LowerLayer
        {
            get { return _lowerLayer; }
            set
            {
                if (_lowerLayer != value)
                {
                    _lowerLayer = value;
                    OnPropertyChanged();
                }
            }
        }

        /// <summary>
        /// Gets or sets the value of the upper layer.
        /// </summary>
        public UpperFieldType UpperLayer
        {
            get { return _upperLayer; }
            set
            {
                if (_upperLayer != value)
                {
                    _upperLayer = value;
                    OnPropertyChanged();
                    OnPropertyChanged(nameof(UpperAndDir));
                }
                else if (value == UpperFieldType.Explosion)
                {
                    _upperLayer = UpperFieldType.Empty;
                    OnPropertyChanged();
                    OnPropertyChanged(nameof(UpperAndDir));
                    _upperLayer = UpperFieldType.Explosion;
                    OnPropertyChanged();
                    OnPropertyChanged(nameof(UpperAndDir));
                }
            }
        }

        /// <summary>
        /// Gets or sets the value of the direction of the entity that may be in the upper layer.
        /// </summary>
        public Direction Dir
        {
            get { return _dir; }
            set
            {
                if (_dir != value)
                {
                    _dir = value;
                    OnPropertyChanged();
                    OnPropertyChanged(nameof(UpperAndDir));
                }
            }
        }

        /// <summary>
        /// Gets the upper layer and it's direction in one property.
        /// </summary>
        public Tuple<UpperFieldType, Direction> UpperAndDir
        {
            get { return new(_upperLayer, _dir); }
        }
    }
}
